/* $Id: test.h,v 1.8 2001/05/06 02:20:08 spj Exp $
** ----------------------------------------------------------------------
** $Log: test.h,v $
** Revision 1.8  2001/05/06 02:20:08  spj
** A nice bugfix and some more documentation.
**
** Revision 1.7  2001/03/30 05:15:07  spj
** Serialize and unserialize now work.  Now I want to figure out how I can measure
** the amount of ram being used by the application-- it's pretty obvious to me
** now that during stress testing I've got a huge memory leak which is slowing
** things down... but more importantly... I've got a huge memory leak.
**
** Question is, do I write the fork() code before I fix the leak? If I write the fork() code
** it will likely hide the leakage. --spj
**
** Revision 1.6  2001/03/25 19:44:35  spj
** Rather ugly changes, but at the moment au_pop_netstring passes it's unit
** tests, so I'm closer to finishing au_test_unserialize.
** --spj
**
** Revision 1.5  2001/03/10 19:23:09  spj
** Split off netstrings from the test.c module into au-netsring.c.
**
** Removing the bit of hand-coded test suite setup and making it initialize
** from the same static array of test cases caused the stress-test to turn
** up a bug in au_new_test where it wasn't initizlizing the elapsed times
** for the test. I guess I omitted or ignored that test in the stress
** rounds.  Now stress tests take a little bit of time which means that
** I should have paid more attention to the errors. :)
**
** Revision 1.4  2001/03/10 06:03:36  spj
** Reduced amount of "hand coded" test setup in c-unit-suite because
** it was becoming onerous to repeat the code. W/e you find yourself
** copying and pasting code, think about perhaps making your life easier.
** --spj
**
** Revision 1.3  2001/03/08 06:35:50  spj
** Tested netstring functions.  Serialize is tested, unserialize needs to be
** written.
**
** Revision 1.2  2001/02/19 19:26:14  spj
** 0.04 checkin.  Some report reformats and new unit tests for au_remove_test
** and au_delete_test.  TODO: au_delete_testcase.
** --spj
**
** Revision 1.1.1.1  2001/02/18 20:42:40  spj
** First checkin.  'c-unit' works but self-testing suite needs to be extended
** to cover more of its functionality. "self-testing" recursively with your
** own self-testing framework gets confusing sometimes. ;) --spj
**
**
*/

#ifndef AUTOUNIT_C_TEST_H
#define AUTOUNIT_C_TEST_H

#include <glib.h>

typedef struct {
  GString *testcase_name;
  GString *testcase_status;
  gboolean testcase_run;
  gint     testcase_pct_complete;
  gdouble  testcase_seconds_elapsed;
  gulong   testcase_useconds_elapsed;

  gpointer *setup_fp; /* cast to autounit_test_setup_fp_t */
  gpointer *teardown_fp; /* cast to autounit_test_teardown_fp_t */
  GSList *tests;
} autounit_testcase_t;

typedef struct {
  autounit_testcase_t *testcase;

  GString *test_name;
  gboolean test_run;
  GString *test_status;
  gint     failed_assertions;
  gdouble  test_seconds_elapsed;
  gulong   test_useconds_elapsed;
  // function pointer to test function
  gpointer *test_fp; /* cast to autounit_test_fp_t */
  gboolean do_fork;
} autounit_test_t;

typedef struct {
  gint     *round;
  gint     modulo;
} autounit_stress_report_t;


typedef gint (*autounit_test_fp_t)(autounit_test_t *t);
typedef gint (*autounit_test_setup_fp_t)(autounit_test_t *t);
typedef gint (*autounit_test_teardown_fp_t)(autounit_test_t *t);


autounit_test_t* au_new_test(GString *test_name,
			     autounit_test_fp_t test_fp);
autounit_testcase_t* au_add_test(autounit_testcase_t *testcase, autounit_test_t *test);
void au_run_test(autounit_test_t *test);
void au_run_test_fork(autounit_test_t *test,autounit_stress_report_t *status);
void             au_run_stress_test(autounit_test_t *test,autounit_stress_report_t *report);
GString *au_test_serialize(autounit_test_t *test);
autounit_test_t *au_test_unserialize(GString *test_ser);

void             au_run_stress_test(autounit_test_t *test,autounit_stress_report_t *report);
void au_delete_test(autounit_test_t *t);
void au_remove_test(autounit_testcase_t *tc, autounit_test_t *t);

autounit_testcase_t* au_new_testcase(GString *testcase_name,
				 autounit_test_setup_fp_t setup_fp,
				 autounit_test_teardown_fp_t teardown_fp);
void au_set_fork_mode(autounit_test_t *t, gboolean mode);
gint             au_run_testcase(autounit_testcase_t *testcase);
gint             au_run_stress_testcase(autounit_testcase_t *testcase,
					gint rounds,gint modulo);
void au_delete_testcase(autounit_testcase_t *t);

#define __au_assert_fail(file,line,msg) \
				   file ":" \
				   #line \
				   ":FAIL:" \
				   msg "\n"
#define au_assert(test,err_msg,expr) (void)((expr) ? 0 : \
                   (test->test_status = \
                   g_string_append(test->test_status, \
				   __au_assert_fail \
                     (__FILE__,__LINE__,err_msg)),test->failed_assertions++)) 

     /* Utility functions used internally by Autounit */
GString *au_gstring_to_netstring(GString *s);
GString *au_netstring_to_gstring(GString *ns);
GString *au_gboolean_to_netstring(gboolean b);
gboolean au_nestring_to_gboolean(GString *ns);
gint au_nestring_to_gint(GString *ns);
gdouble au_netstring_to_gdouble(GString *ns);
gulong au_nestring_to_gulong(GString *ns);
GString *au_gint_to_netstring(gint gi);
GString *au_gdouble_to_netstring(gdouble gd);
GString *au_gulong_to_netstring(gulong gul);
GString *au_read_netstring(int fd);
GString *au_pop_netstring(GString *ns);

#ifndef HAVE_STRNDUP
#include <string.h>
char * strndup(const char * s, size_t n);
#endif

#endif  /* AUTOUNIT_C_TEST_H */

